/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.RequestImportEditor");
dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.core.util.IRmpsConstants");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.importer.ImportClient");

dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("rmps.core.group.GroupClient");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "RequestImportEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.importer", "RequestImportEditorMessages");

var UAKEY = "com.ibm.xtools.rmps.web.importer.page_RequestImportEditor001";
var SNAPSHOT_NAME_UAKEY = "com.ibm.xtools.rmps.web.importer.text_RequestImportEditor001";

var GroupClient = rmps.core.group.GroupClient;

var connectOnInput = function(obj, widget, method) {
	if (dojo.isMoz || dojo.isOpera) {
		widget.connect(obj, "oninput", method);
	} else {
		widget.connect(obj, "onkeydown", method);
		widget.connect(obj, "onkeyup", method);
		widget.connect(obj, "onpaste", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "oncut", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "onblur", method);
	}
};

var createRequiredNode = function() {
	var reqNode = dojo.create("span");
	reqNode.className = "required";
	reqNode.innerHTML = "*";
	reqNode.title = messages.requiredTitle;
	return reqNode;
};

dojo.declare("rmps.importer.RequestImportEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "rmps-importer-request",
	
	uaKey: UAKEY,
	
	// init properties
	isRefreshable: false,
	isReadOnly: true,
	
	_fullImport: false,
	_createBaseline: false,
	_baselineName: null,
	_baselineDescription: null,	
	_requestButton: null,
	
	createContentWidget: function() {
		this._loadCookie();
		return new rmps.importer.RequestImportEditor._Content({editor: this});
	},
	
	addButtons: function(toolbar) {
		this._requestButton = new rmps.ui.SimpleButton({
			label: messages.requestImport,
			onClick: dojo.hitch(this, this._requestImport)
		});
		toolbar.addWidget(this._requestButton);
	},
	
	_requestImport: function() {
		if (this._createBaseline && !this._baselineName) {
			//baseline name is required
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_request_no_baseline_name,
					closable: true
				});
		} else {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.requestingImport
				});
		
			if (this._createBaseline) {
				var baselineName = this._baselineName;
				var baselineDescription = this._baselineDescription;
			}
			var deferred = rmps.importer.ImportClient.requestImport(this.getEditorInput().projectId, this._fullImport, baselineName, baselineDescription);
			deferred.addCallback(this, function() {
				this._refreshStatus();
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: dojo.string.substitute(
						messages.importRequestSuccessful,
						[rmps.core.util.IRmpsConstants.ADMIN_IMPORT_STATUS]),
					closable: true
				});
			});
			deferred.addErrback(this, function(error) {
			    var message, details;
				if (error.status === 412) {
					message = dojo.string.substitute(messages.error_request_no_definition, [rmps.core.util.IRmpsConstants.ADMIN_IMPORT_VIEW_DEFINITION]);
				} else if (error.status === 403) {
					message = messages.error_request_permission;
				} else if (error.status === 409 && this._createBaseline) { 
					message = dojo.string.substitute(messages.error_request_baselineNameConflict, [this._baselineName]);
				} else {
					message = messages.error_request_general;
					details = [{message: dojo.string.substitute(messages['error_request_general.explanation'], [error.status]) + "<br>" + messages['error_request_general.useraction'] }]
				}
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: message,
					details: details,
					closable: true
				});
			});
		}
	},
	
	_refreshStatus: function() {
		var helper = rmps.ui.editor.PageHelper.getInstance();
		var statusEditor = helper.getEditor(
				new rmps.importer.ImportStatusEditorInput({projectId: this.getEditorInput().projectId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	},
	
	_loadCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
			this._fullImport = !!cookie.fullImport;
			this._createBaseline = !!cookie.createBaseline;
		}
	},
	
	_saveCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie.fullImport = this._fullImport;
		//do not save the baseline name and description in the cookie
		cookie.createBaseline = this._createBaseline;
		dojo.cookie(this.declaredClass, dojo.toJson(cookie), {expires: 90});
	}
});

dojo.declare("rmps.importer.RequestImportEditor._Content", [dijit._Widget], {
	
	editor: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		var groupId = null;
		var projectId = jazz.app.context.get().itemId;
		var deferredGroupId = GroupClient.getGroupId(projectId, null, true);
		var definition = null;
							
		deferredGroupId.addBoth(this.editor, function(response){
			this.isReadOnly=true;
			if ( response instanceof Error) {
				if (response.status && (response.status === 410 || response.status === 404)) {
					// if can't find groupId, project area has not been 
					// activated yet, show request import page. 
					this.hideMessage();
					this.contentWidget._init();
					return;
				}else {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						errorObject: response,
						closeable: true
					});
					rmps.core.error.createXhrError(deferredGroupId.ioArgs).dispatch();
					this.isReadOnly = true;
					if (this._requestButton) {
						this._requestButton.setEnabled(false);
					}
					return;
				}
			} else {
				groupId = response;
				var deferredGroupData = GroupClient.getGroup(groupId, null);
				deferredGroupData.addBoth(this, function(response) {
					if ( response instanceof Error) {
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							errorObject: response,
							closeable: true
						});
						rmps.core.error.createXhrError(deferredGroupData.ioArgs).dispatch();
						this.isReadOnly = true;
						if (this._requestButton) {
							this._requestButton.setEnabled(false);
						}
						return;
					} else {
						var group = response[0];
						this._isGroupEditable = group.groupEditable;
						if ( this._isGroupEditable == true) {
							if (this.isReadOnly) {
								dojo.addClass(this.containerNode, "disabled");
							} else {
								this._requestButton.domNode.style.display = "";
								dojo.removeClass(this.containerNode, "disabled");
							}
							if (this._requestButton) {
								this._requestButton.destroy();
							}
							dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) { 
								if (widget.destroyRecursive) {
									widget.destroyRecursive();
								}
							});							
							var widget = new rmps.importer.RequestImportEditor._GroupEditableUploadedSection({editor: this});
							this.containerNode.appendChild(widget.domNode);
							widget.startup();
							return;
						} else {
							this.contentWidget._init();
						}
					}
				});
			}
		});				
	},
	
	_init: function() {
		var widget = new rmps.importer.RequestImportEditor._SubContent({editor :this.editor});
		this.editor.containerNode.appendChild(widget.domNode);
							widget.startup();
		widget.startup();
	},
	
	
});


dojo.declare("rmps.importer.RequestImportEditor._SubContent", [dijit._Widget, dijit._Templated], {
	
	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: '<div>' +
				'<div class="item">' +
					'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="riCb" dojoAttachEvent="onClick: _onReImport"/>' +
					'<label>${messages.fullImport}</label>' +
					'<div class="fullImport-warning" dojoAttachPoint="_fullImportWarning" style="display: none;">${messages.fullImport_warning}</div>' +
				'</div>' +
				'<div class="item">' +
					'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="bCb" dojoAttachEvent="onClick: _onBaseline"/>' +
					'<label>${messages.createBaseline}</label>' +
				'</div>' +
				'<fieldset style="margin-left:20px;" class="baseline-details">' +
					'<legend>${messages.baselineOptions}</legend>' +
					'<table>' +
						'<tr>' +
							'<td style="white-space:nowrap;"><span dojoAttachPoint="_baselineNameLabel">${messages.baselineName}</span></td>' +
							'<td style="width:100%;"><input type="text" dojoAttachPoint="_baselineName"/></td>' +
						'</tr>' +
						'<tr>' +
							'<td style="white-space:nowrap;">${messages.baselineDescription}</td>' +
							'<td style="width:100%;"><textarea rows="4" dojoAttachPoint="_baselineDescription" dojoAttachEvent="onKeyPress: _onDescriptionChange"></textArea></td>' +
						'</tr>' +
					'</table>' +
				'</fieldset>' +				
			'</div>',
	
	// init properties
	editor: null,
	
	_help: null,
	
	postCreate: function() {
		this.riCb.attr("value", this.editor._fullImport);
		this.bCb.attr("value", this.editor._createBaseline);
		this.riCb.domNode.nextSibling.setAttribute("for", this.riCb.id);
		this.bCb.domNode.nextSibling.setAttribute("for", this.bCb.id);
		this._fullImportWarning.style.display = this.editor._fullImport ? "" : "none";
		this._baselineName.disabled = !this.editor._createBaseline;
		//add a "required" star to the name label
		this._baselineNameLabel.parentNode.appendChild(createRequiredNode());
		this._baselineDescription.disabled = !this.editor._createBaseline;
		
		this._help = [];
		this._help.push(new jazz.app.ua.TextHoverHelp({uaKey: SNAPSHOT_NAME_UAKEY, contentNode: this._baselineNameLabel}));
		
		connectOnInput(this._baselineName, this, this._onNameChange);
		connectOnInput(this._baselineDescription, this, this._onDescriptionChange);
	},
	
	_onReImport: function() {
		this.editor._fullImport = !!this.riCb.attr("value");
		this._fullImportWarning.style.display = this.editor._fullImport ? "" : "none";
		this.editor._saveCookie();
	},
	
	_onBaseline: function() {
		this.editor._createBaseline = !!this.bCb.attr("value");
		this._baselineName.disabled = !this.editor._createBaseline;
		this._baselineDescription.disabled = !this.editor._createBaseline;
		this.editor._saveCookie();
	},
	
	_onNameChange: function() {
		this.editor._baselineName = this._baselineName.value;
	},
	
	_onDescriptionChange: function() {
		this.editor._baselineDescription = this._baselineDescription.value;
	},
	
	startup: function() {
		this.inherited(arguments);	

		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	},
	
});

dojo.declare("rmps.importer.RequestImportEditor._GroupEditableUploadedSection", [dijit._Widget, dijit._Templated], {
		
	widgetsInTemplate: true,
	messages: messages,
	
	templateString: '<div>' +
		'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" readOnly="${editor.isReadOnly}" collapsible="false">' +
			'<table class="general">' +
			'<tr><td>' +
			'${messages.editingActive}' +
			'</td></tr>' +
			'</table>' +
		'</div>' +
	'</div>',
		
	postCreate: function() {
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(messages.importDisabled));
		this._section.setPrimaryTitle(titleNode);
	}
});


})();
